package src.elandToCoord;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.StringTokenizer;

import src.lib.Utilities;

public class ElandToCoord
{

	/**
	 * @param args
	 */
	public static void main(String[] args) throws Exception
	{
		
		BufferedReader br = new BufferedReader(new FileReader(args[0]));
		String ofile = args[0]+".coord";
		FileWriter fw = new FileWriter(ofile);
		String line;
		int lineCnt = 0;
		int fcnt = 0;
		while((line = br.readLine())!=null)
		{
			lineCnt++;
			if(lineCnt%10000 == 0) System.out.print(".");
			try
			{
				StringTokenizer st = new StringTokenizer(line,"\t");
				/*outFile.txt is the name of the output file. One line of output per sequence, fields are:
					1. Sequence name (derived from file name and line number if format is not Fasta)
					2. Sequence
					3. Type of match:
					NM - no match found.
					QC - no matching done: QC failure (too many Ns basically).
					RM - no matching done: repeat masked (may be seen if repeatFile.txt was specified).
					U0 - Best match found was a unique exact match.
					U1 - Best match found was a unique 1-error match.
					U2 - Best match found was a unique 2-error match.
					R0 - Multiple exact matches found.
					R1 - Multiple 1-error matches found, no exact matches.
					R2 - Multiple 2-error matches found, no exact or 1-error matches.
					4. Number of exact matches found.
					5. Number of 1-error matches found.
					6. Number of 2-error matches found.
					Rest of fields are only seen if a unique best match was found (i.e. the match code in field 3 begins with "U").
					7. Genome file in which match was found.
					8. Position of match (bases in file are numbered starting at 1).
					9. Direction of match (F=forward strand, R=reverse).
					10. How N characters in read were interpreted: ("."=not applicable, "D"=deletion, "I"=insertion).
					Rest of fields are only seen in the case of a unique inexact match (i.e. the match code was U1 or U2).
					11. Position and type of first substitution error (e.g. 12A: base 12 was A, not whatever is was in read).
					12. Position and type of first substitution error, as above.
					*/
				String name = st.nextToken();
				StringBuffer seq = new StringBuffer(st.nextToken());
				int error = 0;
				String matchType = st.nextToken();
				if(matchType.equals("NM")) continue;
				else if(matchType.equals("QC")) continue;
				else if(matchType.equals("RM")) continue;
				else if(matchType.equals("R0")) continue;
				else if(matchType.equals("R1")) continue;
				else if(matchType.equals("R2")) continue;
				else if(matchType.equals("U0")) error =0;
				else if(matchType.equals("U1")) error =1;
				else if(matchType.equals("U2")) error =2;
				
				else
				{
					throw new Exception("Invalid match type error:"+matchType);
					
				}
				fcnt++;
				int u0MatchCnt = Integer.parseInt(st.nextToken());
				int u1MatchCnt = Integer.parseInt(st.nextToken());
				int u2MatchCnt = Integer.parseInt(st.nextToken());
				String target = st.nextToken();
				int position = Integer.parseInt(st.nextToken());
				String direction = st.nextToken();
				//String howNisUsed = st.nextToken();
				st.nextToken();
				String error1 = null;
				String error2 = null;
				if(error > 0)
					error1 = st.nextToken();
				if(error == 2)
					error2 = st.nextToken();
				StringBuffer genomicSeq = new StringBuffer(seq.toString());
				int seqLen = seq.length();
				char dir = '+';
				//System.out.println(error1+"|"+error2+"|"+howNisUsed);
				
				if(name.charAt(0) == '>') name = name.substring(1);
				
				if(direction.equals("R"))
				{			
					dir = '-';
				}
				StringBuffer homo = new StringBuffer(seqLen);
				for(int i = 0; i < seqLen; i++) homo.append("|");
				double identity = 100;
				if(dir == '-')
				{
					seq = new StringBuffer(Utilities.reverseCompliment(seq.toString()));
					genomicSeq = new StringBuffer(Utilities.reverseCompliment(genomicSeq.toString()));
					
				}
				if(error > 0 )
				{
					identity = (double)(seqLen-1)/(double)seqLen;
					identity= Math.floor(identity*10000);
					identity/=100;
					int pos = Integer.parseInt(error1.substring(0, error1.length()-1));
					char base = error1.substring(error1.length()-1).charAt(0);
					pos--;
					homo.setCharAt(pos,' ');
					genomicSeq.setCharAt(pos, base);				
				}
				if(error ==2 )
				{
					identity = (double)(seqLen-2)/(double)seqLen;
					identity= Math.floor(identity*10000);
					identity/=100;
					int pos = Integer.parseInt(error2.substring(0, error2.length()-1));
					char base = error2.substring(error2.length()-1).charAt(0);
					pos--;
					homo.setCharAt(pos,' ');
					genomicSeq.setCharAt(pos, base);
				}
				int score = 10*seqLen;
				
				if(matchType.equals("U0"))
				{
					score/=u0MatchCnt;
				}
				else if(matchType.equals("U1"))
				{
					score/=u1MatchCnt;
				}
				else if(matchType.equals("U2")) {
					score/=u2MatchCnt;
				}
				/*
				 * ($ntity,
	                 $direct,
	                 $query,
	 		 $score,
	 		 $hsp->query->start,
	                 $hsp->query->end,
	                 $sname,
	 		 $hsp->hit->start,
	 		 $hsp->hit->end,
	                 $hsp->percent,
			 $hsp->P,
			 $hsp->positive,
			 $hsp->qs,
			 $hsp->ss,
			 $hsp->hs
			 
			 );
				 */
				double P = (100-identity)/100.0d;
				P = (double)((int)(P*1000))/1000.0d;
				String c = identity+":"+dir+":"+name+":"+score+":1"+":"+seqLen+":"+target+":"+position+":"+(position+seqLen-1);
				c+=":"+identity+":"+P+":"+(seqLen-error)+":"+seq.toString()+":"+genomicSeq.toString()+":"+homo.toString()+"\n";
				//CoordHitPlus chp = new CoordHitPlus(c);
				if(position < 0)
				{
					System.out.println("Position is less than 0!");
					System.out.println(c);
					System.out.println(line);
				}
				fw.write(c);
			}
			catch (Exception e)
			{
				System.out.println("EXCEPTION");
				System.out.println("Line "+lineCnt+" :"+line);
				
				throw e;
			}
			
			
			
			
			
		
		}
		System.out.println("\nWrote "+fcnt+" records to "+ofile);
		
		fw.close();
	}

}
